/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * instable.c
 * This file is part of gedit
 *
 * Copyright (C) 2002 Paolo Borelli 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */
 
/*
 * Modified by the gedit Team, 2001-2002. See the AUTHORS file for a 
 * list of people on the gedit Team.  
 * See the ChangeLog files for a list of changes. 
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glade/glade-xml.h>
#include <libgnome/gnome-i18n.h>
#include <libgnome/gnome-help.h>

#include <gedit-plugin.h>
#include <gedit-debug.h>
#include <gedit-menus.h>
#include <gedit-utils.h>

#define MENU_ITEM_LABEL	N_("Insert _Table")
#define MENU_ITEM_PATH	"/menu/Edit/EditOps_4/"
#define MENU_ITEM_NAME	"InsertTablePlugin"	
#define MENU_ITEM_TIP	N_("Insert a table at the current position")


G_MODULE_EXPORT GeditPluginState update_ui (GeditPlugin *plugin, BonoboWindow *window);
G_MODULE_EXPORT GeditPluginState init (GeditPlugin *pd);
G_MODULE_EXPORT GeditPluginState destroy (GeditPlugin *pd);
G_MODULE_EXPORT GeditPluginState activate (GeditPlugin *pd);
G_MODULE_EXPORT GeditPluginState deactivate (GeditPlugin *pd);



typedef struct {
	GtkWidget *dialog;
	GtkWidget *spinbutton_rows;
	GtkWidget *spinbutton_cols;
	GtkWidget *spinbutton_width;
	GtkWidget *spinbutton_rowheight;
} InsertTableDialog;


typedef struct {
	gint rows;
	gint columns;
	gint width;
	gint rowheight;
} table_dim;


static void
insert_table (table_dim *td)
{
	GeditDocument *doc;
	GeditView *view;
	gint colwidth;
	gint r, c, cw, rh;

	GString *table = NULL;

	gedit_debug (DEBUG_PLUGINS, "");

	view = gedit_get_active_view ();
	g_return_if_fail (view != NULL);

	doc = gedit_view_get_document (view);
	g_return_if_fail (doc != NULL);

	g_return_if_fail ((td->rows > 0) &&
	                  (td->columns > 0) &&
	                  (td->width > 0) &&
	                  (td->rowheight > 0));

	colwidth = (((td->width - 1))/(td->columns));

	table = g_string_new("\n");
	
	for (r = 1; r <= td->rows; r++) {

		/* horizontal lines */
		for (c = 1; c <= td->columns; c++) {
			g_string_append_c(table,'+');
			for (cw = 1; cw < colwidth; cw++) {
				g_string_append_c(table, '-');
			}
		}
		g_string_append_c(table, '+');
		g_string_append_c(table, '\n');

		/* cells */
		for (rh = 1; rh <= td->rowheight; rh++) {
			for (c = 1; c <= td->columns; c++) {
				g_string_append_c(table, '|');
				for (cw = 1; cw < colwidth; cw++) {
					g_string_append_c(table, ' ');
				}
			}
			g_string_append_c(table, '|');
			g_string_append_c(table, '\n');
		}			
	}
	/* closing line */
	for (c = 1; c <= td->columns; c++) {
		g_string_append_c(table, '+');
		for (cw = 1; cw < colwidth; cw++) {
			g_string_append_c(table, '-');
		}
	}
	g_string_append_c(table, '+');
	g_string_append_c(table, '\n');

	gedit_document_begin_user_action (doc);
	gedit_document_insert_text_at_cursor (doc, table->str, -1);
	gedit_document_end_user_action (doc);

	g_string_free (table, TRUE);
}


static void
dialog_destroyed (GtkObject *obj,  void **dialog_pointer)
{
	gedit_debug (DEBUG_PLUGINS, "");

	if (dialog_pointer != NULL)
	{
		g_free (*dialog_pointer);
		*dialog_pointer = NULL;
	}	
}


static void
dialog_response_handler (GtkDialog *dlg, gint res_id, InsertTableDialog *dialog)
{
	GError *error = NULL;

	static	table_dim td = {
				.rows=3,
				.columns=3,
				.width=72,
				.rowheight=5
			};

	gedit_debug (DEBUG_PLUGINS, "");

	switch (res_id) {
		case GTK_RESPONSE_OK:
			td.rows = gtk_spin_button_get_value (GTK_SPIN_BUTTON (dialog->spinbutton_rows));
			td.columns = gtk_spin_button_get_value (GTK_SPIN_BUTTON (dialog->spinbutton_cols));
			td.width = gtk_spin_button_get_value (GTK_SPIN_BUTTON (dialog->spinbutton_width));
			td.rowheight = gtk_spin_button_get_value (GTK_SPIN_BUTTON (dialog->spinbutton_rowheight));
			insert_table (&td);
			gtk_widget_destroy (GTK_WIDGET (dialog->dialog));
			break;

		case GTK_RESPONSE_HELP:
			/* FIXME: Choose a better link id */
			gnome_help_display ("gedit.xml", "gedit-use-plugins", &error);
	
			if (error != NULL)
			{
				g_warning (error->message);
				g_error_free (error);
			}
			break;
			
		default:
			gtk_widget_destroy (GTK_WIDGET (dialog->dialog));
	}
}


static InsertTableDialog*
insert_table_dialog_create (void)
{
	static InsertTableDialog *dialog = NULL;

	GladeXML  *gui;
	GtkWindow *window;
	GtkWidget *content;

	gedit_debug (DEBUG_PLUGINS, "");

	window = GTK_WINDOW (gedit_get_active_window ());

	if (dialog != NULL)
	{
		gtk_window_present (GTK_WINDOW (dialog->dialog));
		gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog), window);
		return dialog;
	}

	gui = glade_xml_new (GEDIT_GLADEDIR "inserttable.glade2",
	                     "inserttable_dialog_content", NULL);

	if (!gui) {
		g_warning ("Could not find inserttable.glade2, reinstall gedit.\n");
		return NULL;
	}
	
	dialog = g_new0 (InsertTableDialog, 1);

	dialog->dialog = gtk_dialog_new_with_buttons (_("Insert Table"),
	                                             window,
	                                             GTK_DIALOG_DESTROY_WITH_PARENT,
	                                             GTK_STOCK_CLOSE,
	                                             GTK_RESPONSE_CLOSE,
	                                             GTK_STOCK_HELP,
	                                             GTK_RESPONSE_HELP,
	                                             NULL);

	g_return_val_if_fail (dialog->dialog != NULL, NULL);

	/* Add the update button */
	gedit_dialog_add_button (GTK_DIALOG (dialog->dialog), 
	                         _("_Insert table"), GTK_STOCK_ADD, GTK_RESPONSE_OK);

	content = glade_xml_get_widget (gui, "inserttable_dialog_content");
	g_return_val_if_fail (content != NULL, NULL);

	dialog->spinbutton_rows = glade_xml_get_widget (gui, "spinbutton_rows");
	g_return_val_if_fail (dialog->spinbutton_rows != NULL, NULL);
	
	dialog->spinbutton_cols = glade_xml_get_widget (gui, "spinbutton_cols");
	g_return_val_if_fail (dialog->spinbutton_cols != NULL, NULL);
	
	dialog->spinbutton_width = glade_xml_get_widget (gui, "spinbutton_width");
	g_return_val_if_fail (dialog->spinbutton_width != NULL, NULL);
	
	dialog->spinbutton_rowheight = glade_xml_get_widget (gui, "spinbutton_rowheight");
	g_return_val_if_fail (dialog->spinbutton_rowheight != NULL, NULL);
	
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog->dialog)->vbox),
	                    content, FALSE, FALSE, 0);

	gtk_dialog_set_default_response (GTK_DIALOG (dialog->dialog),
	                                 GTK_RESPONSE_OK);

	g_signal_connect (G_OBJECT (dialog->dialog), "destroy",
			   G_CALLBACK (dialog_destroyed), &dialog);

	g_signal_connect (G_OBJECT (dialog->dialog), "response",
	                  G_CALLBACK (dialog_response_handler), dialog);

	g_object_unref (gui);

	gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);

	gtk_widget_show (dialog->dialog);

	return dialog;
}


static void
insert_table_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
{
	InsertTableDialog *dialog = NULL;

	dialog = insert_table_dialog_create();
	if (dialog == NULL) 
	{
		g_warning ("Could not create Insert Table dialog");
		return;
	}
}


G_MODULE_EXPORT GeditPluginState
update_ui (GeditPlugin *plugin, BonoboWindow *window)
{
	BonoboUIComponent *uic;
	
	gedit_debug (DEBUG_PLUGINS, "");	
	g_return_val_if_fail (window != NULL, PLUGIN_ERROR);

	uic = gedit_get_ui_component_from_window (window);

	gedit_menus_set_verb_sensitive (uic, "/commands/" MENU_ITEM_NAME, TRUE);

	return PLUGIN_OK;
}


G_MODULE_EXPORT GeditPluginState
init (GeditPlugin *pd)
{
	/* initialize */
	gedit_debug (DEBUG_PLUGINS, "");
		
	return PLUGIN_OK;
}


G_MODULE_EXPORT GeditPluginState
destroy (GeditPlugin *plugin)
{
	gedit_debug (DEBUG_PLUGINS, "");

	return PLUGIN_OK;
}


G_MODULE_EXPORT GeditPluginState
activate (GeditPlugin *pd)
{
	GList *top_windows;
	gedit_debug (DEBUG_PLUGINS, "");

	top_windows = gedit_get_top_windows ();
	g_return_val_if_fail (top_windows != NULL, PLUGIN_ERROR);

	while (top_windows)
	{
		gedit_menus_add_menu_item (BONOBO_WINDOW (top_windows->data),
		                           MENU_ITEM_PATH, MENU_ITEM_NAME,
		                           MENU_ITEM_LABEL, MENU_ITEM_TIP, NULL,
		                           insert_table_cb);

		pd->update_ui (pd, BONOBO_WINDOW (top_windows->data));

		top_windows = g_list_next (top_windows);
	}

	return PLUGIN_OK;
}


G_MODULE_EXPORT GeditPluginState
deactivate (GeditPlugin *pd)
{
	gedit_menus_remove_menu_item_all (MENU_ITEM_PATH, MENU_ITEM_NAME);

	return PLUGIN_OK;
}

